@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
+ *   Copyright (C) 2010 by Kai Dombrowe <just89@gmx.de>                    *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -18,26 +18,26 @@
  ***************************************************************************/
 
 // own
-#include <kdeversion.h> // krazy:exclude=includes
-#if (KDE_VERSION >= KDE_MAKE_VERSION(4,3,64))
-    #include "mainwindow.h"
-#else
-    #include "mainwindow_4_3.h" // moc workaround
-#endif
+#include "mainwindow.h"
 #include "frame/frame.h"
 #include <recorditnow.h>
 #include "recorditnowpluginmanager.h"
-#include "configdialog.h"
+#include "config/configdialog.h"
 #include "libs/upload/abstractuploader.h"
 #include "recordermanager.h"
 #include "encodermanager.h"
-#include "uploadmanager.h"
 #include "cursorwidget.h"
 #include "application.h"
-#include "mouseconfig.h"
-#include "zoomview.h"
+#include "config/mouseconfig.h"
+#include "zoom/zoomview.h"
 #include "timeline/timeline.h"
 #include "timeline/timelinedock.h"
+#include "upload/uploadwizard.h"
+#include "keyboard/keyboarddock.h"
+#include "config/keyboardconfig.h"
+#include "keymonmanager.h"
+#include "config/frameconfig.h"
+#include "zoom/zoomdock.h"
 
 // Qt
 #include <QtGui/QX11Info>
@@ -47,6 +47,7 @@
 #include <QtGui/QPainter>
 #include <QtGui/QStackedLayout>
 #include <QtGui/QDockWidget>
+#include <QtCore/QDir>
 
 // KDE
 #include <kicon.h>
@@ -65,11 +66,13 @@
 #include <ksqueezedtextlabel.h>
 #include <kactioncategory.h>
 #include <knotification.h>
+#include <ktoolbar.h>
 
 // X11
 #include <X11/Xlib.h>
 
 
+typedef QPair<QString, QSize> Size;
 MainWindow::MainWindow(QWidget *parent)
     : KXmlGuiWindow(parent),
     m_grabber(0)
@@ -86,7 +89,7 @@ MainWindow::MainWindow(QWidget *parent)
 
     timeUpButton->setIcon(KIcon("arrow-up"));
     timeDownButton->setIcon(KIcon("arrow-down"));
-    soundCheck->setIcon(KIcon("preferences-desktop-sound"));
+    soundCheck->setIcon("preferences-desktop-sound");
     deleteButton->setIcon(KIcon("list-remove"));
     playButton->setIcon(KIcon("media-playback-start"));
 
@@ -101,8 +104,9 @@ MainWindow::MainWindow(QWidget *parent)
     connect(outputRequester, SIGNAL(textChanged(QString)), this, SLOT(outputFileChanged(QString)));
 
     m_tray = 0;
-    m_zoom = 0;
     m_timelineDock = 0;
+    m_keyboardDock = 0;
+    m_zoomDock = 0;
 
     m_timer = new QTimer(this);
     connect(m_timer, SIGNAL(timeout()), this, SLOT(tick()));
@@ -125,16 +129,12 @@ MainWindow::MainWindow(QWidget *parent)
     connect(m_encoderManager, SIGNAL(finished(QString)), this,
             SLOT(encoderFinished(QString)));
 
-    m_uploadManager = new UploadManager(this, m_pluginManager);
-    connect(m_uploadManager, SIGNAL(finished(QString)), this, SLOT(uploaderFinished(QString)));
-    connect(m_uploadManager, SIGNAL(status(QString)), this, SLOT(pluginStatus(QString)));
-
 
     m_statusLabel = new KSqueezedTextLabel(this);
     connect(m_statusLabel, SIGNAL(linkActivated(QString)), this, SLOT(linkActivated(QString)));
     statusBar()->addPermanentWidget(m_statusLabel, 1);
 
-    setupTimeline();
+    setupDocks();
     setupTray();
     setupGUI();
 
@@ -174,17 +174,12 @@ MainWindow::~MainWindow()
     delete m_timer;
     delete m_recorderManager;
     delete m_encoderManager;
-    delete m_uploadManager;
     delete m_pluginManager;
 
     if (m_cursor) {
         delete m_cursor;
     }
 
-    if (m_zoom) {
-        delete m_zoom;
-    }
-
 }
 
 
@@ -217,7 +212,7 @@ KAction *MainWindow::getAction(const QString &name)
 
     KAction *action = static_cast<KAction*>(actionCollection()->action(name));
     if (!action) {
-        if (name == "upload" || name == "box") {
+        if (name == "box") {
             action = new KActionMenu(this);
             if (name == "upload") {
                 static_cast<KActionMenu*>(action)->setDelayed(false);
@@ -278,33 +273,18 @@ void MainWindow::setupActions()
     frameMoveAction->setIcon(KIcon("transform-move"));
     connect(frameMoveAction, SIGNAL(triggered(bool)), m_frame, SLOT(setMoveEnabled(bool)));
 
-    KAction *frameRes1Action = getAction("frame_640");
-    frameRes1Action->setData(QSize(640, 480));
-    frameRes1Action->setText("640 x 480 (4:3 SD)");
-    connect(frameRes1Action, SIGNAL(triggered()), this, SLOT(resolutionActionTriggered()));
-
-    KAction *frameRes2Action = getAction("frame_800");
-    frameRes2Action->setData(QSize(800, 600));
-    frameRes2Action->setText("800 x 600");
-    connect(frameRes2Action, SIGNAL(triggered()), this, SLOT(resolutionActionTriggered()));
-
-    KAction *frameRes3Action = getAction("frame_1024");
-    frameRes3Action->setData(QSize(1024, 768));
-    frameRes3Action->setText("1024 x 768");
-    connect(frameRes3Action, SIGNAL(triggered()), this, SLOT(resolutionActionTriggered()));
-
-    KAction *frameRes4Action = getAction("frame_1280");
-    frameRes4Action->setData(QSize(1280, 720));
-    frameRes4Action->setText("1280 x 720 (16x9 HD)");
-    connect(frameRes4Action, SIGNAL(triggered()), this, SLOT(resolutionActionTriggered()));
-
     boxAction->addAction(frameMoveAction);
     boxAction->addSeparator();
-    boxAction->addAction(frameRes1Action);
-    boxAction->addAction(frameRes2Action);
-    boxAction->addAction(frameRes3Action);
-    boxAction->addAction(frameRes4Action);
 
+    QList<Size> sizes;
+    if (Settings::firstStart()) {
+        sizes = FrameConfig::defaultSizes();
+        Settings::self()->setFirstStart(false);
+        FrameConfig::writeSizes(sizes, Settings::self()->config());
+    } else {
+        sizes = FrameConfig::readSizes(Settings::self()->config());
+    }
+    frameSizesChanged(sizes);
 
     KAction *fullAction = getAction("recordFullScreen");
     fullAction->setText(i18n("Record the entire Screen"));
@@ -317,24 +297,15 @@ void MainWindow::setupActions()
     uploadAction->setIcon(KIcon("recorditnow-upload-media"));
     uploadAction->setText(i18n("Upload"));
     uploadAction->setShortcut(Qt::CTRL+Qt::Key_U, KAction::DefaultShortcut);
-
-
-    KAction *zoomAction = getAction("zoom");
-    zoomAction->setObjectName("RecordItNow_zoom");
-    zoomAction->setIcon(KIcon("page-zoom"));
-    zoomAction->setText(i18n("Zoom"));
-    zoomAction->setGlobalShortcut(KShortcut(Qt::META+Qt::CTRL+Qt::Key_Z));
-    connect(zoomAction, SIGNAL(triggered()), this, SLOT(triggerZoom()));
+    connect(uploadAction, SIGNAL(triggered()), this, SLOT(upload()));
 
     KAction *zoomInAction = getAction("zoom-in");
-    zoomInAction->setObjectName("RecordItNow_zoom_in");
     zoomInAction->setIcon(KIcon("zoom-in"));
     zoomInAction->setText(i18n("Zoom in"));
     zoomInAction->setGlobalShortcut(KShortcut(Qt::META+Qt::CTRL+Qt::Key_Plus));
     connect(zoomInAction, SIGNAL(triggered()), this, SLOT(zoomIn()));
 
     KAction *zoomOutAction = getAction("zoom-out");
-    zoomOutAction->setObjectName("RecordItNow_zoom_out");
     zoomOutAction->setIcon(KIcon("zoom-out"));
     zoomOutAction->setText(i18n("Zoom out"));
     zoomOutAction->setGlobalShortcut(KShortcut(Qt::META+Qt::CTRL+Qt::Key_Minus));
@@ -352,10 +323,6 @@ void MainWindow::startRecord()
     setState(Recording);   
     m_recorderManager->startRecord(backendCombo->currentText(), m_recordData);
 
-    if (m_timelineDock && m_recorderManager->hasFeature("TimelineEnabled", backendCombo->currentText())) {
-        m_timelineDock->timeline()->start();
-    }
-
 }
 
 
@@ -393,12 +360,10 @@ void MainWindow::stopRecord()
     } else {
         m_recorderManager->stop();
         m_encoderManager->stop();
-        m_uploadManager->stop();
-        if (m_timelineDock) {
-            m_timelineDock->timeline()->stop();
-        }
     }
 
+    initRecordWidgets(false);
+
 }
 
 
@@ -439,11 +404,11 @@ void MainWindow::recordCurrentWindow()
     int rootX, rootY, winX, winY;
 
     XGrabServer(QX11Info::display());
-    XQueryPointer(QX11Info::display(), QX11Info::appRootWindow(), &root, &child,
+    XQueryPointer(QX11Info::display(), QX11Info::appRootWindow(x11Info().appScreen()), &root, &child,
                   &rootX, &rootY, &winX, &winY, &mask);
 
     if (child == None) {
-        child = QX11Info::appRootWindow();
+        child = QX11Info::appRootWindow(x11Info().appScreen());
     }
 
     XUngrabServer(QX11Info::display());
@@ -484,7 +449,8 @@ void MainWindow::recordFullScreen()
     if (desktop) {
         m_recordData.winId = desktop->winId();
     } else {
-        m_recordData.geometry = kapp->desktop()->screenGeometry(screen);
+        KMessageBox::error(this, i18n("An internal error occurred!"));
+        return;
     }
 
     startTimer();
@@ -532,7 +498,7 @@ void MainWindow::initRecorder(AbstractRecorder::Data *d)
     d->overwrite = Settings::overwrite();
     d->workDir = Settings::workDir().path();
 
-    pluginStatus("");
+    pluginStatus(""); // clear the status bar
 
     if (Settings::hideOnRecord()) {
         if (Settings::tray()) {
@@ -560,7 +526,6 @@ void MainWindow::setupTray()
 
     if (Settings::tray()) {
         if (!m_tray) {
-#if (KDE_VERSION >= KDE_MAKE_VERSION(4,3,64))
             m_tray = new KStatusNotifierItem(this);
             m_tray->setStatus(KStatusNotifierItem::Active);
             m_tray->setCategory(KStatusNotifierItem::ApplicationStatus);
@@ -568,14 +533,6 @@ void MainWindow::setupTray()
             m_tray->setToolTip("recorditnow", i18n("RecordItNow"), "");
             connect(m_tray, SIGNAL(activateRequested(bool,QPoint)), this,
                     SLOT(trayActivated(bool,QPoint)));
-#else
-            m_tray = new KSystemTrayIcon(this);
-            m_tray->setIcon(KIcon("recorditnow"));
-            connect(m_tray, SIGNAL(activated(QSystemTrayIcon::ActivationReason)), this,
-            SLOT(trayActivated(QSystemTrayIcon::ActivationReason)));
-            m_tray->show();
-#endif
-
 
             KMenu *context = new KMenu(this);
             context->addAction(getAction("record"));
@@ -585,11 +542,6 @@ void MainWindow::setupTray()
             context->addAction(getAction("box"));
             context->addAction(getAction("recordWindow"));
             context->addAction(getAction("recordFullScreen"));
-#if (KDE_VERSION >= KDE_MAKE_VERSION(4,3,64))
-#else
-            context->addSeparator();
-            context->addAction((QAction*)KStandardAction::quit(kapp, SLOT(quit()), actionCollection()));
-#endif
             m_tray->setContextMenu(context);
         }
     } else {
@@ -606,20 +558,7 @@ void MainWindow::setTrayOverlay(const QString &name)
 {
 
     if (m_tray) {
-#if (KDE_VERSION >= KDE_MAKE_VERSION(4,3,64))
         m_tray->setOverlayIconByName(name);
-#else
-        QPixmap icon = KIcon("recorditnow").pixmap(KIconLoader::SizeSmallMedium,
-                                                     KIconLoader::SizeSmallMedium);
-        if (!name.isEmpty()) {
-            QPixmap overlay = KIcon(name).pixmap(KIconLoader::SizeSmallMedium/2,
-                                                 KIconLoader::SizeSmallMedium/2);
-            QPainter p(&icon);
-            p.drawPixmap(icon.width()-overlay.width(), icon.height()-overlay.height(), overlay);
-            p.end();
-        }
-        m_tray->setIcon(icon);
-#endif
     }
 
 }
@@ -642,6 +581,8 @@ void MainWindow::setState(const State &newState)
             getAction("box")->setChecked(m_frame->isVisible());
             getAction("options_configure")->setEnabled(true);
             getAction("upload")->setEnabled(true);
+            getAction("zoom-in")->setEnabled(true);
+            getAction("zoom-out")->setEnabled(true);
             fpsSpinBox->setEnabled(m_recorderManager->hasFeature("FPS", recorder));
             soundCheck->setEnabled(m_recorderManager->hasFeature("Sound", recorder));
             centralWidget()->setEnabled(true);
@@ -652,7 +593,8 @@ void MainWindow::setState(const State &newState)
                 show(); // necessary to apply window flags
                 move(p);
             }
-            setupRecordWidgets(false);
+            initRecordWidgets(false);
+            initKeyMon(false);
             break;
         }
     case Timer: {
@@ -666,8 +608,10 @@ void MainWindow::setState(const State &newState)
             getAction("box")->setEnabled(false);
             getAction("options_configure")->setEnabled(false);
             getAction("upload")->setEnabled(false);
+            getAction("zoom-in")->setEnabled(true);
+            getAction("zoom-out")->setEnabled(true);
             centralWidget()->setEnabled(false);
-            setupRecordWidgets(true);
+            initKeyMon(true);
             break;
         }
     case Recording: {
@@ -681,6 +625,8 @@ void MainWindow::setState(const State &newState)
             getAction("box")->setEnabled(false);
             getAction("options_configure")->setEnabled(false);
             getAction("upload")->setEnabled(false);
+            getAction("zoom-in")->setEnabled(true);
+            getAction("zoom-out")->setEnabled(true);
             centralWidget()->setEnabled(false);
             timerLcd->display(0);
             if (Settings::stayOnTop()) {
@@ -690,6 +636,7 @@ void MainWindow::setState(const State &newState)
                 show(); // necessary to apply window flags
                 move(p);
             }
+            initRecordWidgets(true);
             break;
         }
     case TimerPaused: {
@@ -704,6 +651,8 @@ void MainWindow::setState(const State &newState)
             getAction("box")->setEnabled(false);
             getAction("options_configure")->setEnabled(false);
             getAction("upload")->setEnabled(false);
+            getAction("zoom-in")->setEnabled(true);
+            getAction("zoom-out")->setEnabled(true);
             centralWidget()->setEnabled(false);
             break;
         }
@@ -719,6 +668,8 @@ void MainWindow::setState(const State &newState)
             getAction("box")->setEnabled(false);
             getAction("options_configure")->setEnabled(false);
             getAction("upload")->setEnabled(false);
+            getAction("zoom-in")->setEnabled(true);
+            getAction("zoom-out")->setEnabled(true);
             centralWidget()->setEnabled(false);
             break;
         }
@@ -733,21 +684,26 @@ void MainWindow::setState(const State &newState)
             getAction("box")->setEnabled(false);
             getAction("options_configure")->setEnabled(false);
             getAction("upload")->setEnabled(false);
+            getAction("zoom-in")->setEnabled(true);
+            getAction("zoom-out")->setEnabled(true);
             centralWidget()->setEnabled(false);
-            setupRecordWidgets(false);
+            initRecordWidgets(false);
+            initKeyMon(false);
             break;
         }
-    case Upload: {
-            setTrayOverlay("arrow-up-double");
+        case Config: {
+            setTrayOverlay("configure");
             getAction("pause")->setIcon(KIcon("media-playback-pause"));
             getAction("record")->setEnabled(false);
             getAction("pause")->setEnabled(false);
-            getAction("stop")->setEnabled(true);
+            getAction("stop")->setEnabled(false);
             getAction("recordWindow")->setEnabled(false);
             getAction("recordFullScreen")->setEnabled(false);
             getAction("box")->setEnabled(false);
             getAction("options_configure")->setEnabled(false);
             getAction("upload")->setEnabled(false);
+            getAction("zoom-in")->setEnabled(false);
+            getAction("zoom-out")->setEnabled(false);
             centralWidget()->setEnabled(false);
             break;
         }
@@ -776,16 +732,7 @@ void MainWindow::pluginStatus(const QString &text)
 void MainWindow::recorderFinished(const QString &error, const bool &isVideo)
 {
 
-    if (m_zoom) {
-        triggerZoom();
-    }
-
-    if (m_timelineDock) {
-        m_timelineDock->timeline()->stop();
-    }
-
     if (!error.isEmpty()) {
-        KMessageBox::error(this, error);
         pluginStatus(error);
         setState(Idle);
     } else if (Settings::encoderName().isEmpty() || !Settings::encode() || !isVideo) {
@@ -807,7 +754,6 @@ void MainWindow::encoderFinished(const QString &error)
 {
 
     if (!error.isEmpty()) {
-        KMessageBox::error(this, error);
         pluginStatus(error);
     } else {
         playFile(false);
@@ -822,22 +768,32 @@ void MainWindow::encoderFinished(const QString &error)
 void MainWindow::configure()
 {
 
+    setState(Config);
+
+    if(ConfigDialog::showDialog("settings")) {
+        return;
+    }
+
     ConfigDialog *dialog = new ConfigDialog(this, actionCollection(), m_pluginManager);
-    connect(dialog, SIGNAL(dialogFinished()), this, SLOT(dialogFinished()));
+    connect(dialog, SIGNAL(dialogFinished()), this, SLOT(configDialogFinished()));
     dialog->show();
 
 }
 
 
-void MainWindow::dialogFinished()
+void MainWindow::configDialogFinished()
 {
 
-    setupTimeline();
+    setState(Idle);
+
+    setupDocks();
     setupTray();
     updateRecorderCombo();
-    updateUploaderMenu();
     triggerFrame(false); // update KAction
 
+    // update frame menu
+    frameSizesChanged(FrameConfig::readSizes(Settings::self()->config()));
+
 }
 
 
@@ -858,38 +814,6 @@ void MainWindow::updateRecorderCombo()
 }
 
 
-void MainWindow::setupRecordWidgets(const bool &start)
-{
-
-    // mouse
-    if (start) {
-        const QString recorder = backendCombo->currentText();
-        if (Settings::showActivity() && m_recorderManager->hasFeature("LEDEnabled", recorder)) {
-            if (m_cursor) {
-                return; // timer was paused
-            }
-            m_cursor = static_cast<Application*>(kapp)->getCursorWidget(this);
-            connect(m_cursor, SIGNAL(error(QString)), this, SLOT(cursorError(QString)));
-
-            m_cursor->setButtons(MouseConfig::getButtons());
-            m_cursor->setSize(QSize(Settings::cursorWidgetSize(), Settings::cursorWidgetSize()));
-            m_cursor->setNormalColor(Settings::defaultColor());
-            m_cursor->setUseKeyMon(Settings::useKeyMon(), Settings::keyMonDevice());
-            m_cursor->setMode(Settings::led() ? CursorWidget::LEDMode : CursorWidget::CircleMode);
-            m_cursor->setOpacity(Settings::cursorOpacity());
-
-            m_cursor->start();
-        }
-    } else {
-        if (m_cursor) {
-            m_cursor->stop();
-            m_cursor = 0;
-        }
-    }
-
-}
-
-
 void MainWindow::playFile(const bool &force)
 {
 
@@ -912,6 +836,7 @@ void MainWindow::removeFile()
     QFile file(outputRequester->text());
     if (!file.exists()) {
         KMessageBox::sorry(this, i18nc("%1 = file", "%1 no such file!", file.fileName()));
+        outputFileChanged(outputRequester->text());
         return;
     }
 
@@ -951,18 +876,6 @@ void MainWindow::recorderStateChanged(const AbstractRecorder::State &newState)
 }
 
 
-void MainWindow::uploaderFinished(const QString &error)
-{
-
-    if (!error.isEmpty()) {
-        KMessageBox::error(this, error);
-        pluginStatus(error);
-    }
-    setState(Idle);
-
-}
-
-
 void MainWindow::startTimer()
 {
 
@@ -1021,7 +934,6 @@ void MainWindow::lcdDown()
 
 }
 
-#if (KDE_VERSION >= KDE_MAKE_VERSION(4,3,64))
 void MainWindow::trayActivated(const bool &active, const QPoint &pos)
 {
 
@@ -1032,25 +944,19 @@ void MainWindow::trayActivated(const bool &active, const QPoint &pos)
     }
 
 }
-#else
-void MainWindow::trayActivated(const QSystemTrayIcon::ActivationReason &reason)
-{
 
-    if (reason == QSystemTrayIcon::Trigger &&
-        isVisible() &&
-        (state() == Recording || state() == Paused) &&
-        m_recorderManager->currentState() == AbstractRecorder::Record) {
-        stopRecord();
-    }
-
-}
-#endif
 
 void MainWindow::backendChanged(const QString &newBackend)
 {
 
     outputRequester->clear(); // call textChanged()
-    outputRequester->setText(m_recorderManager->getDefaultFile(newBackend));
+
+    QString file = Settings::videoDir().path();
+    if (!file.endsWith(QDir::separator())) {
+        file.append(QDir::separator());
+    }
+    file.append(m_recorderManager->getDefaultFile(newBackend));
+    outputRequester->setText(file);
     setState(Idle); // update actions/widgets
 
 }
@@ -1059,6 +965,7 @@ void MainWindow::backendChanged(const QString &newBackend)
 void MainWindow::aboutToQuit()
 {
 
+    KeyMonManager::self()->stop();
     deleteLater(); // bug???
 
 }
@@ -1072,31 +979,6 @@ void MainWindow::pluginsChanged()
     }
     // recorder
     updateRecorderCombo();
-    // upload
-    updateUploaderMenu();
-
-}
-
-
-void MainWindow::updateUploaderMenu()
-{
-
-    KActionMenu *action = static_cast<KActionMenu*>(getAction("upload"));
-    KMenu *menu = action->menu();
-    if (!menu) {
-        menu = new KMenu(this);
-        action->setMenu(menu);
-    }
-    menu->clear();
-
-    foreach (const UploadData &d, m_uploadManager->getUploader()) {
-        KAction *uploadAction = new KAction(this);
-        uploadAction->setText(d.first);
-        uploadAction->setIcon(d.second);
-        uploadAction->setData(d.first);
-        connect(uploadAction, SIGNAL(triggered()), this, SLOT(upload()));
-        menu->addAction(uploadAction);
-    }
 
 }
 
@@ -1104,13 +986,8 @@ void MainWindow::updateUploaderMenu()
 void MainWindow::upload()
 {
 
-    if (state() == Upload) {
-        return;
-    }
-
-    setState(Upload);
-    QAction *uploadAction = static_cast<QAction*>(sender());
-    m_uploadManager->startUpload(uploadAction->data().toString(), outputRequester->text(), this);
+    UploadWizard *wizard = new UploadWizard(this);
+    wizard->show();
 
 }
 
@@ -1169,35 +1046,11 @@ void MainWindow::saveNewToolbarConfig()
 }
 
 
-void MainWindow::triggerZoom()
-{
-
-    if (m_zoom) {
-        delete m_zoom;
-        m_zoom = 0;
-    } else {
-        if (state() == Recording) {
-            m_zoom = new ZoomView(this);
-            m_zoom->setSize(QSize(Settings::zoomWidth(), Settings::zoomHeight()));
-            m_zoom->setFactor(Settings::zoomFactor());
-            m_zoom->setFollowMouse(Settings::zoomFollow());
-            if (Settings::zoomHighQuality()) {
-                m_zoom->setQuality(ZoomView::High);
-            }
-            m_zoom->show();
-        }
-    }
-
-}
-
-
 void MainWindow::zoomIn()
 {
 
-    if (!m_zoom) {
-        triggerZoom();
-    } else {
-        m_zoom->setFactor(m_zoom->factor()+1);
+    if (m_zoomDock) {
+        m_zoomDock->zoomIn();
     }
 
 }
@@ -1206,14 +1059,14 @@ void MainWindow::zoomIn()
 void MainWindow::zoomOut()
 {
 
-    if (m_zoom) {
-        m_zoom->setFactor(m_zoom->factor()-1);
+    if (m_zoomDock) {
+        m_zoomDock->zoomOut();
     }
 
 }
 
 
-void MainWindow::setupTimeline()
+void MainWindow::setupDocks()
 {
 
     if (Settings::showTimeLine()) {
@@ -1231,6 +1084,40 @@ void MainWindow::setupTimeline()
         }
     }
 
+    if (Settings::enableKeyboard()) {
+        if (!m_keyboardDock) {
+            m_keyboardDock = new KeyboardDock(this);
+            addDockWidget(Qt::BottomDockWidgetArea, m_keyboardDock);
+        }
+        m_keyboardDock->init(KeyboardConfig::readConfig(Settings::self()->config()));
+    } else {
+        if (m_keyboardDock) {
+            removeDockWidget(m_keyboardDock);
+            delete m_keyboardDock;
+            m_keyboardDock = 0;
+        }
+    }
+
+    if (Settings::zoomDockEnabled()) {
+        if (!m_zoomDock) {
+            m_zoomDock = new ZoomDock(this);
+            addDockWidget(Qt::BottomDockWidgetArea, m_zoomDock);
+        }
+        m_zoomDock->setFactor(Settings::zoomFactor());
+        m_zoomDock->setFPS(Settings::zoomFps());
+        if (Settings::zoomHighQuality()) {
+            m_zoomDock->setQuality(ZoomView::High);
+        } else {
+            m_zoomDock->setQuality(ZoomView::Low);
+        }
+    }else {
+        if (m_zoomDock) {
+            removeDockWidget(m_zoomDock);
+            delete m_zoomDock;
+            m_zoomDock = false;
+        }
+    }
+
 }
 
 
@@ -1244,17 +1131,6 @@ void MainWindow::timeLineFinsihed()
 }
 
 
-void MainWindow::cursorError(const QString &message)
-{
-
-    KNotification *notification = new KNotification("error", this, KNotification::CloseOnTimeout);
-    notification->setText(message);
-    notification->setPixmap(KIcon("dialog-error").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium));
-    notification->sendEvent();
-
-}
-
-
 void MainWindow::resolutionActionTriggered()
 {
 
@@ -1264,10 +1140,109 @@ void MainWindow::resolutionActionTriggered()
 }
 
 
-#if (KDE_VERSION >= KDE_MAKE_VERSION(4,3,64))
-    #include "mainwindow.moc"
-#else
-    #include "mainwindow_4_3.moc"
-#endif
+void MainWindow::frameSizesChanged(const QList< QPair<QString, QSize> > &sizes)
+{
+
+    KActionMenu *boxAction = static_cast<KActionMenu*>(getAction("box"));
+    foreach (QAction *act, boxAction->menu()->actions()) {
+        if (act->data().isNull()) {
+            continue;
+        }
+        boxAction->removeAction(act);
+    }
+
+    foreach (const Size &s, sizes) {
+        KAction *frameResAction = new KAction(this);
+        frameResAction->setData(s.second);
+        frameResAction->setText(s.first);
+        frameResAction->setProperty("CleanText", s.first);
+        connect(frameResAction, SIGNAL(triggered()), this, SLOT(resolutionActionTriggered()));
+
+        boxAction->addAction(frameResAction);
+    }
+
+}
 
 
+void MainWindow::initRecordWidgets(const bool &start)
+{
+
+    const QString recorder = backendCombo->currentText();
+    // mouse
+    if (start) {
+        if (Settings::showActivity() && m_recorderManager->hasFeature("MouseEnabled", recorder)) {
+            if (m_cursor) {
+                return; // timer was paused
+            }
+            m_cursor = new CursorWidget(this);
+            connect(m_cursor, SIGNAL(error(QString)), this, SLOT(errorNotification(QString)));
+
+            m_cursor->setButtons(MouseConfig::getButtons(Settings::self()->config()));
+            m_cursor->setSize(QSize(Settings::cursorWidgetSize(), Settings::cursorWidgetSize()));
+            m_cursor->setDevice(Settings::keyMonDevice());
+            m_cursor->setMode(Settings::led() ? CursorWidget::LEDMode : CursorWidget::CircleMode);
+            m_cursor->setOpacity(Settings::cursorOpacity());
+            m_cursor->setShowAlways(Settings::mouseWidgetAlwaysVisible());
+
+            m_cursor->start();
+        }
+    } else {
+        if (m_cursor) {
+            m_cursor->stop();
+            m_cursor = 0;
+        }
+    }
+
+    // timeline
+    if (m_timelineDock) {
+        if (start) {
+            if (m_recorderManager->hasFeature("TimelineEnabled", recorder)) {
+                m_timelineDock->timeline()->start();
+            }
+        } else {
+            m_timelineDock->timeline()->stop();
+        }
+    }
+
+}
+
+
+void MainWindow::initKeyMon(const bool &start)
+{
+
+    QStringList keyMonDevs;
+    const bool mouseFeature = m_recorderManager->hasFeature("MouseEnabled", backendCombo->currentText());
+    const bool keyboardFeature = m_recorderManager->hasFeature("KeyboardEnabled", backendCombo->currentText());
+
+    if (Settings::showActivity() && mouseFeature) {
+        keyMonDevs.append(Settings::keyMonDevice());
+    }
+
+    if (m_keyboardDock && keyboardFeature) {
+        keyMonDevs.append(Settings::keyboardDevice());
+    }
+
+    if (!keyMonDevs.isEmpty() && start) {
+        if (!KeyMonManager::self()->start(keyMonDevs) && !KeyMonManager::self()->error().isEmpty()) {
+            errorNotification(KeyMonManager::self()->error());
+        }
+    } else if (!start) {
+        KeyMonManager::self()->stop();
+    }
+
+}
+
+
+void MainWindow::errorNotification(const QString &error)
+{
+
+    KNotification *notification = new KNotification("error", this, KNotification::CloseOnTimeout);
+    notification->setText(error);
+    notification->setPixmap(KIcon("dialog-error").pixmap(KIconLoader::SizeMedium, KIconLoader::SizeMedium));
+    notification->sendEvent();
+
+}
+
+
+#include "mainwindow.moc"
+
